from __future__ import (unicode_literals, print_function, absolute_import)
from builtins import str
from iotile.core.hw.hwmanager import HardwareManager
from iotile.core.exceptions import ArgumentError

class FlashBoardStep (object):
    """A Recipe Step used to flash firmware directly

    Currently only supports using jlink to flash. Meant to bootstrap firmware

    Args:
        file (str): Firmware file name to flash
        port (str): Port used. Currently only allows jlink port connections.
    """
    FILES = ['file']

    def __init__(self, args):
        if 'file' not in args:
            raise ArgumentError("FlashBoardStep required parameters missing", required=["file"], args=args)

        if 'port' not in args:
            raise ArgumentError("FlashBoardStep Parameter Missing", parameter_name='port', args=args)

        if not args['port'].startswith('jlink'):
            raise ArgumentError("FlashBoardStep is currently only possible through jlink", invalid_port=args['port'])

        self._file = args['file']
        self._port = args['port']

    def run(self):
        with HardwareManager(port=self._port) as hw:
            debug = hw.debug()
            debug.flash(self._file)


"""Test our update script generation and parsing."""

import pytest
from iotile.core.exceptions import ArgumentError, DataError
from iotile.core.hw.update.records import *
from iotile.core.hw import UpdateScript


def test_basic_script_parsing():
    """Make sure we can parse an update script."""

    reflash = ReflashTileRecord(1, bytearray(100), 0x1000)
    unknown = UnknownRecord(128, bytearray(15))

    script = UpdateScript([reflash, unknown])

    encoded = script.encode()

    script2 = UpdateScript.FromBinary(encoded)

    assert len(script2.records) == 2

    with pytest.raises(DataError):
        UpdateScript.FromBinary(encoded, allow_unknown=False)

    assert script == script2
    assert not script != script2

    script3 = UpdateScript([reflash, unknown, unknown])

    assert script3 != script2
